#
#Definition of elements for DocGenerator.rb
#
if __FILE__ == $0
	require 'DocGenerator' 
else
	#
	#Definition of HTML-Tags (Sublasses of Element).
	ATTR_LANG		= Attribute.create( [ :html ], ['en', 'de'] )
	#
	HTML_ATTR_CORE	= {	:id	=> Attribute.create( [ :html ] ),
						:class => Attribute.create( [ :html ] ),
						:style=> Attribute.create( [ :html ] ),
						:title => Attribute.create( [ :html ] ) }
	HTML_ATTR_I18N	= {	:lang	=> ATTR_LANG,
						:dir	=> Attribute.create( [ :html ], ['ltr', 'rtl'] )
					}
	HTML_ATTR_EVENTS = {	:onclick		=> Attribute.create( [ :html ] ), 
						:ondblclick		=> Attribute.create( [ :html ] ), 
						:onmousedown	=> Attribute.create( [ :html ] ),
						:onmouseup	=> Attribute.create( [ :html ] ),
						:onmouseover	=> Attribute.create( [ :html ] ),
						:onmousemove	=> Attribute.create( [ :html ] ),
						:onmouseout	=> Attribute.create( [ :html ] ), 
						:onkeypress	=> Attribute.create( [ :html ] ),
						:onkeyup		=> Attribute.create( [ :html ] ), 
						:onkeydown	=> Attribute.create( [ :html ] )
				}
	HTML_ATTR_ALL = {}
	HTML_ATTR_ALL.update(HTML_ATTR_I18N)
	HTML_ATTR_ALL.update(HTML_ATTR_EVENTS)
	HTML_ATTR_ALL.update(HTML_ATTR_CORE)
	
	HTML_ATTR_ALIGN	= Attribute.create( [ :html ], ['left', 'center', 'right', 'justify', 'char'] )
	HTML_ATTR_VALIGN	= Attribute.create( [ :html ], ['bottom', 'top', 'middle', 'baseline'] )
end

#
#Main document control.
#
Element.create( [:head], {}, true, 
	{	:htmltag => 'head',
		:latex	=> '#{@content}',
	} )
Element.create( [:body],	{
		:bgcolor	=> Attribute.create( [ :html ] ),
		:text		=> Attribute.create( [ :html ] ),
		:link		=> Attribute.create( [ :html ] ),
		:vlink	=> Attribute.create( [ :html ] ),
		:alink	=> Attribute.create( [ :html ] ),
	}.update(HTML_ATTR_CORE.dup.update(HTML_ATTR_I18N)),
	true, 
	{	:htmltag	=> 'body',
		:latex	=> '#{@content}',
	} )

Element.create( [:latexdummy ], {},true, 
	{	:htmltag	=> nil,
		:latex	=> '#{@content}'
	} )
Element.create( [:htmldummy ], {},true, 
	{	:html	=> '#{@content}',
		:latex	=> ''
	} )

#
#Headings and sectioning
#
sect = Struct.new( 'Sections', :keys, :html, :tex )
[
	sect.new( [:chapter],			'', 'chapter'),	#fixme: neue HTML-Datei?
	sect.new( [:h1, :section],		'h1', 'section'),
	sect.new( [:h2, :subsection],		'h2', 'subsection'),
	sect.new( [:h3, :subsubsection],	'h3', 'subsubsection'),
	sect.new( [:h4, :paragraph],		'h4', 'paragraph'),
	sect.new( [:h5, :subparagraph],	'h5', 'subparagraph'),
	sect.new( [:minisec],			'h5', 'minisec'),
].each{ |s|
	Element.create( s.keys,	HTML_ATTR_ALL,	true, 
		{	:htmltag	=> s.html,
			:latex	=> "\n\\#{s.tex}" + '{#{@content}}' + "\n",
			:text	=> "\n\n" + '#{@content}' + "\n"
		} )
}
#~ Element.create( [:h1],	HTML_ATTR_ALL,	true, 
	#~ {	:htmltag	=> 'h1',
		#~ :latex	=> "\n" + '\section{#{@content}}' + "\n"
	#~ } )
#~ Element.create( [:h2],	HTML_ATTR_ALL,	true, 
	#~ {	:htmltag	=> 'h2',
		#~ :latex	=> "\n" + '\subsection{#{@content}}' + "\n"
	#~ } )


#
#List
#
class Environment < Element
	def to_latex_environment( environment, parameter = nil)
		cmd	=	''
		cmd	<<	"\n" if @crbefore
		cmd	<<	"\n\\begin{#{environment}}"
		#~ cmd	<< parameter if parameter
		cmd	<< @content.to_s
		cmd	= cmd.chomp + "\n"
		cmd	<<	"\\end{#{environment}}\n"
		cmd	<<	"\n" if @crafter
		return cmd
	end
	#~ def htmltag()
		#~ return 'table'
	#~ end
	#~ def to_latex( )
		#~ return to_latex_environment( 'itemize' )
	#~ end
end

class ListEnvironment < Environment
	add_attributes( HTML_ATTR_ALL ) 
	#Only items are allowed to be added.
	def << ( item)
		if ! item.is_a?( Element.get(:item) )
			puts "Add non-item to list (#{item.class})"
		end
		@content << item
	end
end
class Itemize < ListEnvironment
	Element.add( [:ul, :itemize],  Itemize)
	def htmltag(); 'ul'; end
	def to_latex( )
		return to_latex_environment( 'itemize' )
	end
end
class Enumerate < ListEnvironment
	Element.add( [:ol, :enumerate],  Enumerate)
	def htmltag(); 'ol'; end
	def to_latex( )
		return to_latex_environment( 'enumerate' )
	end
end
class Description < ListEnvironment
	Element.add( [:description],  Description)
	def to_latex( )
		return to_latex_environment( 'description' )
	end
end
class Item < Element
	Element.add( [:li, :item],  Item)
	add_attributes( 		{
			:text => Attribute.create( [ :latex ] )
		}.update(HTML_ATTR_ALL)
	) 
	def htmltag(); 'li'; end
	def to_latex( )
		cmd	= ''
		cmd	+=	"\n" if @crbefore
		cmd	+=	"\n\\item "
		cmd	+=	"[#{@attr[:text]}] " if @attr[:text].filled?
		cmd	+= @content.to_s
		cmd	+=	"\n" if @crafter
		return cmd
	end
	def to_text()
		return "\n" + "- #{@content}" + "\n"
	end
end


#
#Titlepage
#
#This parameter are better set with Document.title.
#
class Title < Element
	Element.add( [:title],  Title)
	add_attributes( HTML_ATTR_ALL ) 
	add_output( :latex,	'\title{#{@content}}' + "\n" + '\hypersetup{pdftitle={#{@content}}}' + "\n")
	add_output( :html,	'<title>#{@content}</title>' + "\n")
end
Element.create( [:meta], {
		:name	=> Attribute.create( [ :html, :required ], nil, 1 ),
		:content	=> Attribute.create( [ :html, :required ], nil, 2 ),
	}, nil,
	{	
		:htmltag	=> 'meta',
		#~ :html	=> '<meta name="#{@attr[:name]}" content="#{@attr[:content]}">',
		:latex	=> '',
#		:latex	=> "\n" + '\author{#{@content}}\hypersetup{pdfauthor={#{@content}}}' + "\n"
	} )

Element.create( [:author], {}, true,
	{	:html	=> '<meta name="author" content="#{@content}">' + "\n",
		:latex	=> "\n" + '\author{#{@content}}\hypersetup{pdfauthor={#{@content}}}' + "\n"
	} )
Element.create( [:date], {}, true,
	{	:html	=> '<meta name="creation-date" content="#{@content}">' + "\n",
		:latex	=> "\n" + '\date{#{@content}}' + "\n"
	} )
Element.create( [:keywords], {}, true,
	{	:html	=> '<meta name="keywords" content="#{@content}">' + "\n",
		:latex	=> "\n" + '\hypersetup{pdfkeywords={#{@content}}}' + "\n"
	} )
Element.create( [:metadescription], {}, true,
	{	:html	=> '<meta name="description" content="#{@content}">' + "\n",
		:latex	=> "\n" + '\hypersetup{pdfsubject={#{@content}}}' + "\n"
	} )
#~ Element.create( [:hypersetup], {}, true,
	#~ {	:html	=> '<meta name="description" content="#{@content}">',
		#~ :latex	=> "\n" + '\pdfinfo{description=#{@content}}' + "\n"
	#~ } )


Element.create( [:maketitle], {}, false,
	{	:htmltag	=> 'span',	#Empty span
		:html	=> "<!-- maketitle -->\n",
		:latex	=> "\n" + '\maketitle' + "\n"
	} )
Element.create( [:tableofcontents], {}, false,
	{	:htmltag	=> 'span',	#Empty span
		:html	=> "<!-- tableofcontents -->\n",
		:latex	=> "\n" + '\tableofcontents' + "\n"
	} )

#
#Generell paragraph handling.
#
Element.create( [:par, :p],	HTML_ATTR_ALL,	true, 
	{	:htmltag	=>	'p',
		:latex		=>	"\n\n" + '#{@content}' + "\n\n",
		:text		=>	"\n" + '#{@content}' + "\n",
	} )
#~ par.add_output( :html,	'<p>#{@attr[:content]}</p>')
#~ par.add_output( :latex,	'#{@content}\par')

Element.create( [:br, 'br/', 'br', :newline], {},false, 
	{	:htmltag	=> 'br',
		:latex	=> '\newline' + "\n"
	} )
Element.create( [:vfill], {},false, 
	{	:htmltag	=> nil,
		:latex	=> '\vfill '
	} )
Element.create( [:hfill], {},false, 
	{	:htmltag => nil,
		:latex	=> '\hfill '
	} )

#
#Grouping
#
Element.create( [:span, :group],	HTML_ATTR_CORE,	true, 
	{	:htmltag	=>	'span',
		:latex	=>	"\n" + '\bgroup' + 
					'#{@content}' + "\n" +
					"\n" + '\egroup'
	} )
Element.create( [:div, :Group],	HTML_ATTR_CORE,	true, 
	{	:htmltag	=> 'div',
		:latex	=>	"\n" + '\begingroup' + 
					'#{@content}' + "\n" +
					"\n" + '\endgroup'
	} )

#Make a separator lin in code
Element.create( [:sepline],	{},	false, 
	{	:html	=>	"\n<!--" +	'-' * 50 + " -->\n",
		:latex		=>	"\n%" +	'-' * 50 + "\n"
	} )
#Make a comment in code
Element.create( [:comment],	{},	true, 
	{	:htmltag	=> '<!-- #{@content} -->',
		:latex	=>	"\n" + '% ------- #{@content} -------' + "\n"
	} )

#
#Links
#
#Better: \hypertarget, hyperlink (pdf)
Element.create( [ :url ], 
	HTML_ATTR_ALL,
	true,
	{
		:html	=> '<a href="#{@content}">#{@content}</a>',
		:latex	=> '\url{#{@content}}',
	} )
Element.create( [ :a, :href ], 
	{	:href		=> Attribute.create( [ :html, :required ] ),
		:hreflang	=> ATTR_LANG,
		:alt		=> Attribute.create( [ :html ] ),
		:longdesc	=> Attribute.create( [ :html ], [String] ),
		:value	=> Attribute.create( [ :html ] ),
		:shape	=> Attribute.create( [ :html ] ),
		:coords	=> Attribute.create( [ :html ] ),
		:target	=> Attribute.create( [ :html ], [  '_blank', '_parent', '_self', '_top' ] ),
	}.update( HTML_ATTR_ALL ),
	true,
	{
		:htmltag	=> 'a',
		:latex	=> '\href{#{@attr[:href]}}{#{@content}}',
	} )

Element.create( [ :label ], 
	{	:name		=> Attribute.create( [ :html, :required ] ),
	}.update( HTML_ATTR_ALL ),
	true,
	{
		:htmltag	=> 'a',
		:latex	=> '\label{#{@attr[:name]}}#{@content}',
	} )

Element.create( [ :hypertarget ], 
	{	:href		=> Attribute.create( [ :html, :required ] ),
		:alt		=> Attribute.create( [ :html ] ),
		:longdesc	=> Attribute.create( [ :html ] ),
		:value	=> Attribute.create( [ :html ] ),
		:shape	=> Attribute.create( [ :html ] ),
		:coords	=> Attribute.create( [ :html ] ),
	}.update( HTML_ATTR_ALL ),
	true,
	{
		#~ :htmltag	=> 'a',
		:latex	=> '\hypertarget{#{@attr[:href]}}{#{@content}}',
	} )

#

class Usepackage < Element
	#Make the key known to the complete key-list of elements.
	Element.add( [:usepackage],  Usepackage)
	#~ add_attributes( HTML_ATTR_ALL ) 
	add_attributes( { :option =>  Attribute.create( ) } ) 
	
	#With usepackage additional command my be defined.
	#Here we load this special 
	def << ( content )
		@content << content
		#Load new makros
		makrodefinitions = 'DocGenerator_' + content
		begin
			require makrodefinitions 
		rescue LoadError 
		end
	end	
	
	add_output( :latex,	'\usepackage[#{@attr[:option]}]{#{@content}}' )
	#Redefine standard output. 
	#Options are given, if an option is avalaible.
	def to_latex()
		cmd = '\usepackage'
		opt = @attr[:option].content
		cmd += '[' + opt.join(',') + ']' if opt.size > 0
		cmd += "{#{@content}}\n"
		return cmd
	end
	def to_html()
		return ''
	end
end	#Usepackage



#~ :em	=> {}, #complete it
#~ :sub	=> all.update({}), #complete it
#~ :sup	=> all.update({}), #complete it
#~ :pre	=> {}, #complete it
#~ :code	=> {}, #complete it
Element.create( [:font], {
			:size		=>  Attribute.create([:html], [ Fixnum ]),
		},true, 
	{	:htmltag => 'font',
		#~ :latex	=> '\tiny ' #+ "\n"
	} )
#Fixme: Jeweils :tinytex fr ungeklammerte
Element.create( [:tiny ], {},true, 
	{	#:html => '<span style="font-size:xx-small">#{@content}</span>',
		:latex	=> '{\tiny  #{@content}}' #+ "\n"
	} )
Element.create( [:scriptsize ], {},true, 
	{	#:htmltag => 'br',
		:latex	=> '{\scriptsize  #{@content}}' #+ "\n"
	} )
Element.create( [:footnotesize ], {},true, 
	{	#:htmltag => 'br',
		:latex	=> '{\footnotesize  #{@content}}' #+ "\n"
	} )
#fixme \small oder {\small ..}	
Element.create( [:small ], {},true, 
	{	#:html => '<span style="font-size:small">#{@content}</span>',
		:htmltag	=> 'small',
		:latex	=> '{\small #{@content}}' #+ "\n"
	} )
Element.create( [:normalsize ], {},true, 
	{	#:htmltag => 'br',
		:latex	=> '{\normalsize  #{@content}}' #+ "\n"
	} )
Element.create( [:large ], {},true, 
	{	#:htmltag => 'br',
		:latex	=> '{\large  #{@content}}' #+ "\n"
	} )
Element.create( [:Large ], {},true, 
	{	#:htmltag => 'br',
		:latex	=> '{\Large  #{@content}}' #+ "\n"
	} )
Element.create( [:huge ], {},true, 
	{	#:htmltag => 'br',
		:latex	=> '{\huge  #{@content}}' #+ "\n"
	} )
Element.create( [:Huge ], {},true, 
	{	#:htmltag => 'br',
		:latex	=> '{\Huge  #{@content}}' #+ "\n"
	} )

Element.create( [:textbf, :strong ], {},true, 
	{	:htmltag	=> 'strong',
		:latex	=> '\textbf{#{@content}}'
	} )
Element.create( [:emph ], {},true, 
	{	:htmltag	=> 'em',
		:latex	=> '\emph{#{@content}}'
	} )

	
Element.create( [:newpage ], {},false, 
	{	#:htmltag => '',
		:latex	=> '\newpage ' + "\n"
	} )
Element.create( [:twocolumn ], {},false, 
	{	#:htmltag => '',
		:latex	=> '\twocolumn ' + "\n"
	} )
Element.create( [:center ], HTML_ATTR_CORE, true, 
	{	:htmltag	=> 'center',
		:latex	=> "\n\\begin{center}\n" + '#{@content}' + "\n\\end{center}\n"
	} )
Element.create( [:quote, :blockquote ], HTML_ATTR_CORE, true, 
	{	:htmltag	=> 'blockquote',
		:latex	=> "\n\\begin{quote}\n" + '#{@content}' + "\n\\end{quote}\n"
	} )
#fixme: TeX kein content, HTML hat content
Element.create( [:centering ], {},false, 
	{	:htmltag	=> 'center',
		:latex	=> '\centering ' + "\n"
	} )
Element.create( [:landscape ], {}, true, 
	{	#:htmltag	=> 'center',
		:latex	=> "\n\\begin{landscape}\n" + '#{@content}' + "\n\\end{landscape}\n"
	} )

#~ :meta=> { :content=>nil, :name=>nil }, #complete it
#~ :hr	=> {}, #complete it
#~ :img	=> all.update({:width=>nil, :src=>nil,:alt=>nil, :longdesc=>nil, :border=>nil,:name=>nil, :align=>[:top,:middle,:bottom,:left,:right], :hspace=>nil, :vspace => nil, :usemap=>nil}),#complete it
#~ :nobr	=> {}, #complete it
#~ :link	=> {}, #complete it
#~ :center	=> {}, #complete it
#~ :map	=> all.update({}), #complete it
#~ :area	=> all.update({}), #complete it
#~ :option=> {:value=>nil}, #complete it
#~ :select=> events.update({}), #complete it

Element.create( :input, {}, true )


Element.create( [:img, :includegraphics],
	{
		:src		=>  Attribute.create([:html, :latex, :required], [ String ]),
		:alt		=>  Attribute.create([:html], [String]	),
		:style		=>  Attribute.create([:html], [String]	),
		:width	=>  Attribute.create([:html, :texkeyval], [Integer]	),
		:height	=>  Attribute.create([:html, :texkeyval], [Integer]	),
		:border	=>  Attribute.create([:html], [Integer]	),
	}, false,
	{	#:html => '<span style="font-size:xx-small">#{@content}</span>',
		:htmltag	=> 'img',
		:latex	=> '\includegraphics[#{texkeyval}]{#{@attr[:src]}}' 
	}
)
