#
#Definition of elements for DocGenerator.rb
#
if __FILE__ == $0
	require 'DocGenerator' 
end

#Tabulars
#fixme:  check on attributes. columnsdescription
#~ :table=> {:border=>nil, :width=>nil, :cellpadding=>nil, :cellspacing=>nil},#complete it
#~ :colgroup => {}, #comlpete it
class Tabular < Element
	#Make the key known to the complete key-list of elements.
	Element.add( [:tabular],  Tabular )
	#~ add_attributes( HTML_ATTR_ALL ) 
	#Problem: Nicht weitervererbt
	add_attributes( {
		:columndescription	=>  Attribute.create( [:latex] ) ,
		:columns			=>  Attribute.create( [:required], [ Fixnum ] ) ,
		:border			=>  Attribute.create( [:html] ) ,
		:bgcolor			=> Attribute.create( [ :html ] ),
		:width			=> Attribute.create( [ :html ], [ Integer ] ),
		:frame			=> Attribute.create( [ :html ], [ 'box', 'above', 'below', 'hsides', 'vsides', 'lhs', 'rhs' ] ),
		:rules			=> Attribute.create( [ :html ], [ 'none', 'rows', 'cols', 'groups', 'all' ] ),
		:height			=> Attribute.create( [ :html ], [ Integer ] ),
		:cellspacing		=> Attribute.create( [ :html ], [ Integer ] ),
		:cellpadding		=> Attribute.create( [ :html ], [ Integer ] ),
		:summary			=> Attribute.create( [:html] ) ,
	}.update( HTML_ATTR_ALL ) )
	#Only rows are allowed to be added.
	#The row get the number of columns.
	#The row counting does not word, when the row-content is added immediate in element-initialization.
	def << ( row )
		if row.is_a?( Row )
			row.columns = @attr[:columns].to_s.to_i
		else
			puts "Add non-row to tabular (#{row.class})"
		end
		@content << row
	end
	def to_latex()
		cmd	=	''
		cmd	+=	"\n" if @crbefore
		cmd	+=	"\\begin{tabular}{"
		cmd	+=	@attr[:columndescription].content.to_s
		cmd	+=	"}"
		cmd	+= @content.to_s
		cmd	+=	"\\end{tabular}"
		cmd	+=	"\n" if @crafter
		return cmd
	end
	def htmltag()
		return 'table'
	end
end
class Longtable < Tabular
	#Make the key known to the complete key-list of elements.
	Element.add( [:longtable],  Longtable )
	#
	add_attributes( Element.get_attribute_list( Tabular ) )
	def to_latex()
		cmd	=	''
		cmd	+=	"\n" if @crbefore
		cmd	+=	"\\begin{longtable}{"
		cmd	+=	@attr[:columndescription].content.to_s
		cmd	+=	"}"
		cmd	+= @content.to_s
		cmd	+=	"\\end{longtable}"
		cmd	+=	"\n" if @crafter
		return cmd
	end
end
#May only be used insode a tabular. Tabular must set the columns-value. 
#If not, zero columns is expected and an error is thrown.
class Row < Element
	#Make the key known to the complete key-list of elements.
	Element.add( [:row, :tr],  Row )
	#~ :tr	=> {:align=>align},#complete it
	add_attributes( {
			:add_vspace	=>  Attribute.create( [:latex] ) ,
			:bgcolor			=> Attribute.create( [ :html ] ),
			#fixme
			#~ :hline			=>  Attribute.create( [:latex], [true, false] ) ,
		}.update(HTML_ATTR_ALL) ) 
	#Set from tabular
	attr_accessor :columns
	def hline();		@hline = true;		self; end
	def toprule();		@toprule = true;		self; end
	def midrule();		@midrule = true;		self; end
	def bottomrule(); 	@bottomrule = true;	self; end
	#Only inside a longtable
	def endhead();		@endhead = true;	self; end
	#Only inside a longtable
	def endfoot();		@endfoot = true;	self; end
	#Only inside a longtable
	def endfirsthead();	@endfirsthead = true;	self; end
	#Only inside a longtable
	def endlastfoot();	@endlastfoot = true;	self; end
	#Only Colums are allowed to be added.
	def << ( row )
		if ! row.is_a?( Column )
			puts "Add non-column to tabular (#{row.class})"
		end
		@content << row
		#Does not work with multicolumns.
		if !@columns
			puts "Add row to tabular with undefined columns number (#{row.inspect})" 
		elsif @content.size > @columns 
			puts "Tabular with #{@attr[:columns]} columns get #{@content.size} columns" 
		end
	end
	def inspect()
		return "<Class Row:#{id}> @content={@content.inspect}"
	end
	#Each cell is separated by '&'.
	def to_latex()
		cmd	=	''
		cmd	<<	"\n" if @crbefore
		cmd	<< @content.join(' & ') + Regexp.escape('\\' + '\\')
		cmd	<< "[#{@attr[:add_vspace].content.to_s}]" if @attr[:add_vspace].content.size > 0
		cmd	<< '\hline '		if @hline
		cmd	<< '\toprule ' 	if @toprule
		cmd	<< '\midrule ' 	if @midrule
		cmd	<< '\bottomrule '	if @bottomrule
		cmd	<< '\endfirsthead ' if @endfirsthead
		cmd	<< '\endhead ' 	if @endhead
		cmd	<< '\endfoot ' 	if @endfoot
		cmd	<< '\endlastfoot '	if @endlastfoot
		cmd	<<	"\n" if @crafter
		return cmd
	end
	def htmltag()
		return 'tr'
	end
end
#One column in a tabular.
class Column < Element
	#Make the key known to the complete key-list of elements.
#~ :th	=> {:colspan => nil, :rowspan => nil, :align=>align, :valign=>valign, :border=>nil, :width=>nil},#complete it
#~ :td	=> nil,	#like :th
	Element.add( [:col, :column, :td, :th],  Column )
	add_attributes( HTML_ATTR_ALL )
	add_attributes( { 
		:width 	=> Attribute.create( [ :html ] ),
		:align 	=> HTML_ATTR_ALIGN,
		:bgcolor	=> Attribute.create( [ :html ] ),
	} )
	def to_latex()
		cmd	=	''
		cmd	+=	"\n" if @crbefore
		cmd	+=	"#{@content}"
		cmd	+=	"\n" if @crafter
		return cmd
	end
	def htmltag()
#	fixme th for head	
		return 'td'
	end
	def inspect()
		return "<Class Column @content=#{@content.inspect}>"
	end
end
class Multicolumn < Column
	#Make the key known to the complete key-list of elements.
	Element.add( [:multicolumn],  Multicolumn )
	#~ add_attributes( HTML_ATTR_ALL ) 
	add_attributes( { 
		#~ :columns	=>  Attribute.create( [:required], [ Fixnum ] ) ,	#colspan
		:columns	=>  Attribute.create( [:required, :html, :latex], [ Fixnum ] ) ,	#colspan
		:colspan	=>	:columns,
		#~ :columns	=> :colspan,	#define alias
		:pos		=>  Attribute.create( [:latex] ) ,
		:align	=>  HTML_ATTR_ALIGN,
	}.update( HTML_ATTR_ALL ) ) 
	def to_latex()
		cmd	=	''
		cmd	+=	"\n" if @crbefore
		cmd	+= 	"\\multicolumn{#{@attr[:columns]}}{#{@attr[:pos]}}{#{@content}}"
		cmd	+=	"\n" if @crafter
		return cmd
	end
	#~ def htmltag()
		#~ return 'td'
	#~ end
	#~ def to_html()
		#~ cmd	=	''
		#~ cmd	+=	"\n" if @crbefore
		#~ cmd	+= 	"<td colspan=\"#{@attr[:columns]}\">"
		#~ cmd	+= "#{@content}</td>"
		#~ cmd	+=	"\n" if @crafter
		#~ return cmd
	#~ end
	def inspect()
		return "<Class Multicolumn @content=#{@content.inspect}>"
	end
end
