#Usage:
#-Place picture in a directory
#-Add a configuration file with description
#-Start the tool
#-Translate the created TeX-File
#
#Structure of the config-File, see BeamerDoc#commands
#	
#Hinweis:
#	head:\DeclareGraphicsExtensions{.jpeg}
#	head:\DeclareGraphicsRule{.jpeg}{jpg}{.jpeg}{}
#
#Example of a script:
#	require 'C:/usr/Script/MyScripts' if File.exist?('C:/usr/Script/MyScripts.rb')
#	MKPRESOVERWRITE = true
#	require 'mkpres'
#	
#	__END__
#	>>file:
#	>>title:
#	>>runtex:yes
#	>>overwrite:yes
#	>>template:beamerthemetree
#	>>language:ngerman
#	<article>\tableofcontents
#	>>head:\AtBeginSection[] % Do nothing for \section*
#	>>head:{
#	>>head:	\frame<beamer>
#	>>head:	{
#	>>head:	%\frametitle{}
#	>>head:	\tableofcontents[current]
#	>>head:	}
#	>>head:}


#Offen:
# - Muster fr Dateinamen
# - Mix mit berarbeiteter Prsentation?
# - Here-Dokumente auslagern und ein "Publish-Skript" schreiben das die Dokumente einfgt.
# - Handout / Compactfassung
# - direkt bersetzen
# - Bildgrsse erkennen (RMagick Installieren. Und die Methoden rows, columns des Image Objekts benutzen.)

#~ require 'C:/usr/Script/MyScripts' if File.exist?('C:/usr/Script/MyScripts.rb')
require 'DocGenerator'
require 'date'
require 'rtex'

#Define TeX-Kommands
Element.create( :frame, {}, true, {
	:latex => '\frame{#{@content}}'
	} )
Element.create( :frametitle, { }, true, {
	:latex => '\frametitle{#{@content}}'
	} )
#\framePic{filename}{
#	description
#}
#~ Element.create( :framepic, {	:file	=> Attribute.create( [ :obligatory ] ), 
					#~ :link	=> Attribute.create( )
				#~ }, true, {
	#~ :latex => '\framePic{#{@attr[:file]}}{#{@content}}'
	#~ } )
class Framepic < Element
	Element.add( [:framepic],  Framepic)
	add_attributes( {	:file	=> Attribute.create( [ :obligatory ] ), 
					:link	=> Attribute.create( )
				} ) 
	def to_latex()
		result = "\\framePic{#{@attr[:file]}}"
		if @attr[:link].content.size > 0
			result += "[#{@attr[:link]}]"
		end
		result += "{#{@content}}"
		return result
	end
end


Element.create( :includegraphicsJustified, {}, true, {
	:latex => '\includegraphicsJustified{#{@content}}'
	} )



#For usage in another script:
#	doc		= BeamerDoc.new()
#	doc.commands( DATA )
#	doc.save( filename, true )
#Can also be used directly via shell.
class BeamerDoc < Document
	DocumentTemplate.add_template(:beamer, %q|<<prefix>>
		% ----------------------------------------------------------------
		<<head>>
		% ----------------------------------------------------------------
		\begin{document}
		<<body>>
		\end{document}
		% ----------------------------------------------------------------
	|.gsub(/^\t*/, '') )
	DocumentTemplate.add_template(:beamerhead, %q|<<prefix>>
		\documentclass[<<classoptions>>]{beamer}
		%<<theme>>
		\usepackage{babel}
		\usepackage[utf8]{inputenc}
		\usepackage[T1]{fontenc}
		\usepackage{beamertool}
		\usepackage{hyperref}
		% ----------------------------------------------------------------
		<<head>>
	|.gsub(/^\t*/, ''))
	[	:beamerthemebars, :beamerthemeboxes, :beamerthemeclassic,
		:beamerthemelined, :beamerthemeplain, :beamerthemesidebar,
		:beamerthemesidebardark, :beamerthemesidebardarktab,
		:beamerthemesidebartab, :beamerthemesplit,
		:beamerthemesplitcondensed, :beamerthemetree,
		:beamerthemetreebars
	].each{ |theme|
		DocumentTemplate.add_template(theme, DocumentTemplate[:beamerhead].sub(/%<<theme>>/, "\\usepackage{#{theme}}"))
	}
	DocumentTemplate.add_template(:beamerarticle, %q|<<prefix>>
		\documentclass[class=scrartcl,<<classoptions>>]{beamer}
		\usepackage{babel}
		\usepackage{beamerbasearticle}
		\usepackage[article]{beamertool}
		\usepackage[utf8]{inputenc}
		\usepackage[T1]{fontenc}
		\usepackage{hyperref}
		% ----------------------------------------------------------------
		<<head>>
	|.gsub(/^\t*/, ''))

	#Default filename for BeamerDoc#save.
	attr_accessor :filename
	#Default-Extension for the files to be included.
	attr_accessor :extension
	#Stuff which is added to each frame
	attr_reader :eachframe
	#Call pdflatex immediate after the document generation.
	def runtex=( flag = true )
		@pres_runtex = flag
	end
	def initialize( template = :beamerhead )
		super( :beamer )
		@maintemplate = template
		@filename = 'pres.tex'
		@extension = ['', '.png', '.jpg']
		@attachment	= []
		@eachframe	= []
		@classoptions	= []
		@language	= 'ngerman'
	end
	#Analyse the configfile. Structure of the config-File:
	#	>>file:Filename
	#	>>title:...
	#	>>extension:Ext
	#	>>template:Template
	#	>>option:Class option
	#	>>include:dir/config-File
	#	>>head:\usepackage...
	#	<mode>content
	#	!part
	#	*section
	#	**subsection
	#	picture name:title
	#	-item line for picture
	#	+text without item
	#	:title
	#	-text for frame without picture
	#
	#The include-file is taken and interpreted.
	#All >>-commands are ignored (the include should not redefine the filename).
	def commands( com, dir = '' )
		#Initialize
		list		= nil
		item		= nil
		llist		= nil
		iitem		= nil
		content	= nil
		
		com.each{ |line|
			if line =~ /^\s*$/
				next 
			elsif line =~ /^>>(.*):(.*)/
				cmd = $1
				option = $2.chomp
				case cmd
				when 'file'	#set filename
					@filename = option
					@filename  += '.tex' if ! ( /\.tex/ =~ @filename )
				when 'extension'		#set extension
					@extension = ['']
					option.split(';').each{ |ext|
						ext = ".#{ext}" if ext[0..0] != '.'
						@extension << ext
					}
				when 'template'
					@maintemplate = option.to_sym
				when 'option'
					@classoptions << option
				when 'head'
					option.gsub!( //, Date.today.year.to_s )	if option =~/year\s*=\s*\{\}/
					option.gsub!( //, Date.today.month.to_s )	if option =~/month\s*=\s*\{\}/
					option.gsub!( //, Date.today.day.to_s )	if option =~/day\s*=\s*\{\}/
					if option =~/addasattachment/
						@head << "%note = {\n"
						@head << "%\\attachfile[description={#{@title} - Article}]{#{Dir.pwd()}/#{@filename.sub(/\.tex/, '')}_article.pdf}\n"
						@head << "%\\attachfile[description={#{@title}} - Presentation]{#{Dir.pwd()}/#{@filename.sub(/\.tex/, '')}_presentation.pdf}\n"
						@head << "%},\n"
						next
					end
					@head	<< option + "\n"
				when 'eachframe'
					@eachframe	<< option + "\n"
				when 'language'
					@language	= option
				when 'runtex'
					@pres_runtex = true if option =~ /yes/i
				when 'overwrite'
					@overwrite = true if option =~ /yes/i
				when 'include'
					if File.exist?( option )
						@body << "\n% Include #{option} Start\n"
						File.open( option ){ |f| 
							inccom = f.readlines.to_s
							inccom.gsub!( /^>>.*/, '')
							
							commands( inccom.split("\n"), File.dirname( option ) + '/' )
						}
						@body << "\n% Include #{option} End\n"
					else
						"Include #{option} not found"
					end
				when 'title'
					self.title = option
					@body << element( :frame, {}, element( :maketitle ) ).cr
				else
					puts "unsupported #{line}"
				end
			elsif line =~ /^<(.*)>(.*)/
				@body << "\\only<#{$1}>{#{$2}}"
			elsif line[0..0] == '#'
				next
			elsif line[0..0] == '%'
				next if line[1..-1] == "\n"
				txt = line
				if iitem
					iitem	<< txt 
				elsif item
					item	<< txt 
				elsif content
					content << txt 
				else
					@body << txt 
				end
			elsif line[0..1] == "!!"
				puts "Use '>>title:....' instead !!"
			elsif line[0..0] == "!"
				content = list = llist = nil
				@body	<< element( :part, {}, line[1..-1].strip)
			elsif line[0..1] == "**"
				content = list = llist = nil
				@body	<< element( :subsection, {}, line[2..-1].strip)
			elsif line[0..0] == "*" 
				content = list = llist = nil
				@body	<< element( :section, {}, line[1..-1].strip)
			elsif line[0..1] == "--" #add text in items if picture is known
				if list
					if llist == nil
						item << llist = element( :ul )
					end
					llist << iitem = element( :li, {}, line[2..-1].strip )
				else
					puts "Listentry II without target #{line[1..-1].strip}"
				end
			elsif line[0..0] == "-" #add text in items if picture is known
				if content
					llist	= nil
					if list == nil
						content << list = element( :ul ).cr
					end
					list << item = element( :li, {}, line[1..-1].strip )
				else
					puts "Listentry without target #{line[1..-1].strip}"
				end
			elsif line[0..0] == "+" or line[0..0] == ">" #add text if picture is known
				if content
					list	= nil
					item	= nil
					llist	= nil
					iitem	= nil
					content << element( :par, {}, line[1..-1].strip )
				else
					puts "Listentry without target #{line[1..-1].strip}"
				end
			elsif line =~ /(.*?):(.*)/
				picname	= $1
				pictitle	= $2
				list	= nil
				llist	= nil
				item	= iitem = nil
				if picname == ''
					content = element( :frame )
					content << @eachframe
					@body << content.CR
					picdesc = line[1..-1].strip
					content << element( :frametitle, {}, pictitle ).cr if pictitle
				else
					detailpic = false
					if picname.sub!( /!/, '')
						detailpic = true					
					end
					
					picname = dir + picname
					found = false
					@extension.each{ |ext|
						found = true if File.exist?( picname + ext )
						#~ puts "Test #{picname + ext}\t#{found}"
						break if found
					}
					puts "Picture not found: #{picname}" if !found
					
					@body	<< pic = element( :frame ).CR
					pic	<< @eachframe
					pic	<< element( :frametitle, {}, pictitle ).cr if pictitle
	
					pic << content = element( :framepic, { :file => picname } )
					if detailpic
						content[:link] << "pic:#{picname}"  if detailpic
					end
					
					if detailpic
						@attachment << '\frame<handout:0>[plain]{' + "\n"
						@attachment << element( :hypertarget, { :href => "pic:#{picname}" } ).cr
						@attachment << element( :includegraphicsJustified, {}, picname).cr
						@attachment << "}\n"
					end
				end
			else
				puts "Error reading control file: #{line}"
			end
		}
	end	#commands
	
	def save( filename, overwrite = false )
		if @attachment.size > 0

			@body << [ '\only<presentation>{',
				%q|
					\appendix
					\usebackgroundtemplate{}
					%
					% Big Pictures / Plainframes
					\part{Detail Pictures}
					\frame{\partpage}
				|.gsub(/^\t*/, ''),
				@attachment,
				'}%Big Pictures' ]			
		end
		
		presentation = Document.new( @maintemplate )
		presentation.language	= @language
		presentation.runtex	= @pres_runtex
		@classoptions.each{|opt| presentation.add_option( opt ) }		
		presentation.head << "%BibTeX-Entry>\n"
		presentation.head << "%BibTeX-Entry<\n"
		presentation.head << "\\input{#{filename}}"

		filename_pres = filename.sub( /\./, '_Presentation.')
	
		article = Document.new( :beamerarticle )
		article.head << "%BibTeX-Entry>\n"
		article.head << "%BibTeX-Entry<\n"
		article.head << "\\input{#{filename}}\n"
		article.language	= @language
		article.runtex	= @pres_runtex
		filename_art = filename.sub( /\./, '_Article.')
		
		
		changed = false
		changed = true if super( 			filename, overwrite )
		changed = true if presentation.save(	filename_pres, overwrite )
		changed = true if article.save(		filename_art, overwrite )
		
		if @pres_runtex
			if changed
				Document.runtex( filename_pres ) 
				Document.runtex( filename_art ) 
			else
				puts 'No TeX-call (no change in documents)'
			end
		end


	end
	#Flag, for overwriting 
	attr_reader :overwrite
end	#class BeamerDoc

#
#From now on, only when called directly (via shell, not via require)
#
#The constant STARTMKPRES can de set in other script.
if __FILE__ == $0 or defined?( MKPRESOVERWRITE )
require 'GetoptLongWithHelp'

opt	= GetoptLongWithHelp.new(
		[ "--filename",		'-f', GetoptLong::REQUIRED_ARGUMENT ],
		[ "--controlfile",	'-c', GetoptLong::REQUIRED_ARGUMENT ],
		[ "--extension",	'-e', GetoptLong::REQUIRED_ARGUMENT ],
		[ "--recursive",	'-r', GetoptLong::NO_ARGUMENT ],
		#~ [ "--all",			'-a', GetoptLong::NO_ARGUMENT ],
		[ "--overwrite",	'-o', GetoptLong::NO_ARGUMENT ],
		[ "--runtex",		'-i', GetoptLong::NO_ARGUMENT ],
		[ "--template",		'-t', GetoptLong::REQUIRED_ARGUMENT ],
		#~ [ "--ALL",			'-A', GetoptLong::NO_ARGUMENT ]
	)

opt.addHelp('-f',"filename to create\t\t(Default: pres.tex)\n\tCan be overwritten in the control file")
opt.addHelp( "-c",%Q|Control-File (Text-Data to mix up with Pictures)
	Format:
		>>Filename (default pres.tex)
		>>-eExtension
		!!Title
		!Part-title
		*Section-title
		**Subsection-titel
		Picture:Title	
		+Text to the picture
		-item line Description (itemize)
		:Title
		+Text for a frame without picture
		-item line for a frame without picture
	If the title is "NONE", the picture isn't added.|)
opt.addHelp( "-e","Date type of pictures\t(Default: png)")
opt.addHelp( "-r","rekursive all Subdirecories")
#~ opt.addHelp( "-a","Create also handout and compact version")
opt.addHelp( "-o","Overwrite existing file")
#~ opt.addHelp( "-A","Create handout and compact version, but no main")
opt.addHelp( "--template",
	%Q|Define the template for the created presentation.
	Supported:
	- itemize: Picture on the left, on the right an itemize-env
	- full:	One Pic per page
	- includepdf: Take pdf's and make a frame for each page
	|.gsub(/^\t/, '') )
opt.addHelp( "--runtex", "Start TeX and create pdfs")

opt.mkHelp(
%Q|Read all Pictures in a directory and create a beamer.cls-Presentation
How to use:
	- create screenshots (e.g. with snagit)
	- call this script -> File pres.tex is created, 
		each picture is on the left, 
		on the right is an itemize env. for comments
Usage:
mkpres [-h] [-f filename] [-c ctrlfile] [-e .ext]|)


############################
# Start
############################
doc		= BeamerDoc.new()
overwrite	= false
controlfile	= 'files.txt'
if $*.size == 0
	puts "mkpres2.rb (use -h for details)"	
	#~ exit
end

command = []
opt.each{ |option, selection|
	case option
	when '--help'
		puts opt.help( selection )
		exit
	when "--filename"
		command << ">>file:#{selection}"
	when "--controlfile"
		controlfile	= selection
	when "--overwrite"
		overwrite	= true
	when "--extension"
		command << ">>extension:#{selection}"
	when "--template"
		command << ">>template:#{selection}"
	when "--runtex"
		command << ">>runtex\n"
	else
		puts "Invalid switch #{option}"
	end
}


if controlfile and File.exist?( controlfile )
	puts "Use controlfile #{controlfile}"
	f = File.new( controlfile )
	command += f.readlines
	f.close
	
	doc.commands( command )
	doc.save( doc.filename, overwrite )
elsif defined?(DATA)
	puts "Use script-internal commands"
	doc.commands( DATA.readlines )
	doc.save( doc.filename, MKPRESOVERWRITE )
else
	puts "No controlfile found (looked for #{controlfile.inspect})"
end

END{
	puts "End"
	$stdin.gets()	if $stdin.tty? 
}
end	#Call via shell
