#
#This program translates a (La)TeX-File as often as necessary and
#gives a log-overview.
#
#Source on CTAN: 
#http://www.dante.de/CTAN/support/rtex	?
#
#Things to get better:
#-	class Config is platform depending and only defined for Win2K
#	- path and program for each tool
#	- stdout/stderr/Log->Files
#- Error and Warning-Analyses must be enforced	
#- support multibib.sty (bibtex)
#- support multind.sty 
#- Option -c to remove all auxiliary files.
#- analyse htlatex
#- Check in which source-file an error/warning occurs (input-list)
#- Check on which output page an error/warning occurs (input-list)
#
#Some programs with a similar functionalty
#- texify (miktex)
#- latexmk
#- texexec (ConTeXt)
#- rubber (Python) http://ebeffara.free.fr/stuff/rubber/
#- texi2pdf (or texi2dvi)
#- mk Perl script http://www.servalys.nl/tex/mk.html
#
#Usage in programms:
#Call TeX and generate document
#	tex = Chain.new()
#	#run.latex 
#	#run.latex_ps_pdf
#	Logger.level=3
#	tex.file = '??'
#	tex.execute()


require 'C:/usr/Script/MyScripts' if File.exist?('C:/usr/Script/MyScripts.rb')
#require 'GetoptLongWithHelp'
#Class Tempfile for temporary redirection of stdout and stderr
require "tempfile" 
#require "tk" 

#Collect messages, condense them and report them.
#There are two output routines: File (complete) and stdout.
class Logger
	private_class_method :new
	@@logger = nil
	# This class is a singleton
	def Logger.create
		@@logger = new unless @@logger
		return @@logger
	end
	def initialize( )
		@start	= Time.now()
		@log		= "Start Logger #{@start}"	#All logging informations
		@level	= 2
		@lastlevel	= 3
		@texrun	= 1	#counter for number of tex runs
		@endflag	= "END-END"
	end
	# Increase the tex-run counter
	def nextrun()
		@log += "\nEnd of TeX-Run #{@texrun}"
		@texrun += 1
		@log += "\nStart of TeX-Run #{@texrun}"
		if @texrun > Config.maxrun
			print "Run #{@texrun} > maximal runs (#{Config.maxrun}) -> abort"
			puts Config.logfile
			save(Config.logfile)
			STDIN.getc if @@wait
			exit
		end
	end
	def texrun();	@texrun	end
	# Add a comment, if the level requires it.
	def add( level, tool, text )
		if level <= @level
			puts line(tool, text, level)
		end
		@log += "\n" + line(tool, text)
	end
	#Each line starts with the number of texrun and the tool)
	def line( tool, content, level=nil )
		if tool == @endflag #end statistic
		return "#{content}"
		else
		return "#{"%1i" % @texrun}#{if level != nil; "(%1i)" % level end} #{"%-15s" % tool}\t#{content}"
		end
	end
	# Add a complete File to the log (e.g. .log-File)
	def add_file( filename, tool = filename )
		file = File.new( filename )
		file.each_line{ |l|
			@log += "\n" + line( " >" + tool, l.chop!() )
		}
		file.close()
	end
	def add_list( list, tool )
		list.each{ |l|
			@log += "\n" + line(tool, l.chop!() )
		}
	end
	# There are different levels of informations:
	# - 0 No information at all
	# - 1 Tool-Error + Summary information from TeX ( run X, X pages, X errors, x Warnings, y Overfull, y Underfulls)
	# - 2 Level 1 + Summary information from tools
	# - 3 Level 2 + Executed steps + Statistics + Errors
	# - 4 Level 3 + Warnings
	# - 5 Level 4 + hints from process
	# - 6 Level 5 + detailed hints from process
	# - 9 Everything
	# All levels are reported in the Log-File (including log-Files etc.)
	def level=( level )
		@level	= level
	end
	attr_reader :level
	def fileInclude=( fileInclude=true )
		@fileInclude = fileInclude
	end
	def stdoutInclude=( stdoutInclude=true )
		@stdoutInclude = stdoutInclude
	end
	def error( text )
		add(5, "", "Error during run #{@texrun}:\t#{text}")
		puts "\nError rtex.rb: #{text}"
		STDIN.getc if @@wait
		exit 
	end
	def save( filename )
		@level	= 9	#take total endstatistic
		@texrun	= 0
		Tool.endstatistic( @endflag )
		if filename != File.basename( filename, '.log') + '.log'
			puts "#{filename} not written (extension <> '.log')"
			STDIN.getc if @@wait
			exit 
		else
			@log	+= "\nEnd Logger after #{@texrun} runs "
			@log += "#{Time.now()} (needed #{Time.now() - @start} seconds)"
			log = File.new(filename, 'w')
			log.puts( @log )
			log.close()
		end
	end
	def Logger.add( level, tool, text ) ; @@logger.add( level, tool, text ) end	
	def Logger.add_file( filename, tool=filename ) ; @@logger.add_file( filename, tool ) end	
	def Logger.add_list( list, tool ) ; @@logger.add_list( list, tool ) end	
	def Logger.fileInclude=( fileInclude=false ); @@logger.fileInclude= fileInclude end	
	def Logger.nextrun( ) ; @@logger.nextrun() end	
	def Logger.texrun( ) ; @@logger.texrun() end	
	def Logger.error( text ) ; @@logger.error(text) end	
	def Logger.level=( level ) ; @@logger.level = level end
	def Logger.set_level( level ) ; @@logger.level = level end
	def Logger.level() ; @@logger.level() end
end
# Create a default Logger
Logger.create()
#

# Configuration for the tool
# This allows an easy change for other systems.
#
# Contains settings for:
# -Tolerance for reporting over- and underfull boxes
# - Pathes and names for Programms
class Config
	# Tolerances for under/overfull boxes
	@@overfull	= 0		#limit for reporting overfull boxes
	@@underfull	= 0		#limit for reporting underfull boxes
	@@showhyperrefwarning	= true
	@@tmpfile	= {}
	@@maxrun	= 5
	#Set the limit for output in the log file, when an overfull box occurs.
	def Config.overfull=( value );	@@overfull	= value.to_i	end
	#Set the limit for output in the log file, when an underfull box occurs.
	def Config.underfull=( value );	@@underfull	= value.to_i	end
	#Flag if Warnings from hyperref should be shown.
	def Config.showhyperrefwarning=( value );	@@showhyperrefwarning = value end
	def Config.logfile=( value );	@@logfile = value.to_s	end
	#Returns the limit for overfull boxes.
	def Config.overfull( );	@@overfull	end
	#Returns the limit for underfull boxes.
	def Config.underfull( );	@@underfull	end
	def Config.maxrun( );	@@maxrun	end
	def Config.logfile( );		@@logfile	end
	def Config.showhyperrefwarning( );	@@showhyperrefwarning	end
	#
	# Pathes and programs
	#
	def Config.latex(  par = "" )
		return 'latex ' + par
	end
	def Config.pdflatex(  par = "" )
		return 'pdflatex ' + par
	end
	def Config.bibtex(  par = "" )
		return 'bibtex ' + par
	end
	def Config.makeindex( par = "" )
		return 'makeindex ' + par
	end
	def Config.rail(  par = "" )
		return 'rail.exe ' + par
	end
	def Config.dvips(  par = "" )
		return 'dvips ' + par
	end
	def Config.ps2pdf( filename = "")
		cmd = 'gswin32c'
		if filename != ""
			cmd += ' -q -dBATCH -dNOPAUSE -sDEVICE=pdfwrite '
			cmd += "-sOutputFile=#{filename}.pdf -c save pop -f #{filename}.ps" 
		end
		return cmd
	end
	def Config.dvipdfm(  par = "" )
		return 'dvipdfm ' + par
	end
	#Prepare start of htlatex to create HTML-Files.
	def Config.htlatex(  par = "" )
		return 'htlatex ' + par
	end
end #Program

# This class defines some generic methods. 
# This methods must be implemented by each tool which is needed.
# The different tools can be:
# - LaTeX itself (or pdfLaTeX...)
# - makeindex (or yindy...)
# - bibtex
# - rail
# Each tool must implement:
# - rerun? : Return true or false
# - execute: runs the tool, returning true or false. False is a hard error, stopping the further processing
#	This method should call execute_tools (not necessary, if @tools is empty)
# - analyse: Return a hash with number of errors, warnings... errorlist...
# A tool may require other tools (LaTeX need BibTeX, Makeindex...)
# This tools are in array @tools.
class Tool
	@@stdout	= STDOUT.clone()
	@@stderr	= STDERR.clone()
	@@result	= {}
	#
	# The following methods must be redefined by the specific tools
	#
	
	#Each tool requires one input file.
	def initialize( )
		@tools = []
		@id	= self.class.to_s
		#@filename		= filename
		@logInclude	= false	#Add log-Files
		@stdoutInclude	= true	#Add the stdout-Files
		@stderrInclude	= false	#Add the stderr-Files
		@dataChanged	= false	#Flag if data changed (requires new TeX-Run)
	end
	#Return a list with instances of the tool. By default it is one instance. 
	#Some tools determine more instances, e.g. makeindex, if there are multiple indeces.
	def Tool.create( filename, toolclass )
		return [ toolclass.new( filename ) ]
	end
	# Each object gets an ID, default is the class.
	# If a tool has more instances, other names are given (e.g name of index).
	def id(); return @id end
	# This Method defines a check, if an additional run is necessary
	def rerun?( log=nil )
		Logger.error "Method rerun? not redefined in #{self.class}"
	end
	# Execute the tool
	def execute()
		Logger.error "Method execute not redefined in #{self.class}"
		#Logger.add( 4, self.class, "Execute #{@@cmd}")
		#@dataChanged = false	#clear flag
		#cmd = "#{@@cmd}"
		#result = system(cmd)  
		#add_stdout( self.class ) if @stdoutInclude
		#Logger.add_file( "filename" ) if @logInclude
		#return result
	end
	# Method to analyse a log-File (or whatever the tool creates to be analysed)
	# Returns a hash with summary of errors/warnings...
	def analyse()
		Logger.error "Method analyse not redefined in #{self.class}"
		#~ result = {	'Error'		=> 0,
				#~ 'Warning'		=> 0,
				#~ 'Errorlist'	=> [],
				#~ 'Warninglist'	=> [],
				#~ 'sum'		=> "",	#Summary line
				#~ }
	end
	#
	# The following methods must not be redefined by the specific tools
	#
	
	def dataChanged?( )
		return @dataChanged
	end
	# Execute all tools in @tools and return true if the data changed,
	# so an additional call is necessary
	def executeTools()
		result = false
		@tools.each{ |toolclass| 
			toolclass.create( @filename, toolclass ).each{ |tool| 
			if tool.rerun?
				tool.execute()
				@@result[tool.id] = tool.analyse()	#collect last result
				add_stdout( tool.class ) if @stdoutInclude
				if tool.dataChanged? 
					Logger.add( 5, "TeX", "Rerun necessary (#{tool.class()})")
					result = true
				end
			end
			}
		}
		return result	#flag if the data are 
	end
	# Object-method for Class-method (easier to call)
	def put_analyse( result ); Tool.put_analyse( result, self.class ) end
	# Takes the result and displays it. Rules for the display:
	#- First summary if available
	#- Only arguments with a value > 0 (Zero error is not reported)
	#- List with Error/Warnings are attached
	def Tool.put_analyse( result, tool )
		if result['Summary'] != nil
			Logger.add(2, tool, "-> #{key}:\t#{result[key]}")
		end
		result.keys.sort.each{ |key|
			if key == 'Summary' or result[key] == 0
				next
			end
			if /list/ !~ key	#Display no lists
				Logger.add(3, tool, "-> #{key}:\t#{result[key]}")
			end			
			#wrap an Position 80?
			if key == 'Informationlist'
				result['Informationlist'].each{ |err|
					Logger.add(3, tool, "\t\t#{err}")
					}
			elsif key == 'Error'
				result['Errorlist'].each{ |err|
					Logger.add(3, tool, "\t\t#{err}")
					}
			elsif key == 'Overfull'
				begin
					Logger.add(4, tool, "\tWarnings for Overfull boxes (>#{Config.overfull}pt):")
					result['Overfulllist'].each{ |err|	
						Logger.add(4, tool, "\t\t#{err}")
						}
				end	if result['Overfulllist'] and result['Overfulllist'] != []
			elsif key == 'Underfull'
				begin
					Logger.add(4, tool, "\tWarnings for Underfull boxes (>badness #{Config.underfull}):")
					result['Underfulllist'].each{ |err|	
						Logger.add(4, tool, "\t\t#{err}")
						}
				end	if result['Underfulllist'] and result['Underfulllist'] != []
			elsif key == 'Warning'
				result['Warninglist'].each{ |err|
					Logger.add(4, tool, "\t\t#{err}")
					}					
				begin
					Logger.add(4, tool, "\tWarnings from Hyperref:\n")
					result['WarninglistHyper'].each{ |err|	
						Logger.add(4, tool, "\t\t#{err}")
						}
				end	if Config.showhyperrefwarning and result['WarninglistHyper'] and result['WarninglistHyper'] != []
				begin
					Logger.add(4, tool, "\tWarnings for labels:")
					result['Labellist'].each{ |err|	
						Logger.add(4, tool, "\t\t#{err}")
						}
				end	if result['Labellist'] and result['Labellist'] != []
				begin
					Logger.add(4, tool, "\tWarnings for citations:")
					result['Citationlist'].each{ |err|	
						Logger.add(4, tool, "\t\t#{err}")
						}
				end	if result['Citationlist'] and result['Citationlist'] != []
			end
		}
		#Create summary if no one exists
		if result['Summary'] == nil
			sum =	"Errors: #{result['Error']}\t" + 
					"Warnings: #{result['Warning']} " 
					#~ "(hyper: #{result['WarningHyper']})\t"
			if result['Pages'] != nil	#from tool LaTeX
				sum	+=	"Overfull: #{result['Overfull']}\t" + 
						"Underfull: #{result['Underfull']}\t"
				sum +=	"Pages: #{result['Pages']}\t" + 
						"Size: #{result['Size']}\t"
			end
			result['Summary'] = sum
		end
		return result
	end	#put_analyse
	# Write a finishing statistic with all results of the last run.
	def	Tool.endstatistic( endflag )
		Logger.add( 1, endflag, "=" * 50 )
		@@result.keys.sort.each{ | tool |
			Logger.add( 1, endflag, "#{tool}:")
			Tool.put_analyse( @@result[tool], endflag )
		}
		return @@result
	end
	# Generall method to read a file.
	def readLog ( filename = @filename + '.log' )
		return Tool.readLog( filename )
	end
	def Tool.readLog ( filename  )
		if ! File.exist?( filename ); return "" end
		log = File.new(filename, 'r')
		content = log.read()
		log.close()
		return content
	end
	# STDOUT is temporary redirected to a file.
	def stdoutOff()
		Tool.stdoutOff()
	end
	def Tool.stdoutOff()
		@@tmpstdout = Tempfile.new( 'stdout')
		STDOUT.reopen( @@tmpstdout ) 
	end
	# Stdout is coming again to the screen.
	def stdoutOn()
		Tool.stdoutOn()
	end
	def Tool.stdoutOn()
		@@tmpstdout.close()
		STDOUT.reopen( @@stdout  )
	end
	# Add the result of stdout to the log-File
	def get_stdout( )		
		Tool.get_stdout( )		
	end
	def Tool.get_stdout( )	
		@@tmpstdout.open()
		stdout = @@tmpstdout.readlines()
		@@tmpstdout.close()
		return stdout
	end
	def add_stdout( tool )
		Logger.add(9, tool, ">>> Start messages (stdout) #{tool}>>>")
		Logger.add_list( get_stdout(), tool)
		Logger.add(9, tool, "<<< End messages (stdout) #{tool}<<<")
	end
	# STDERR is temporary redirected to a file.
	def stderrOff()
		@@tmpstderr = Tempfile.new( 'stderr')
		STDERR.reopen( @@tmpstderr ) 
	end
	# Stdout is coming again to the screen.
	def stderrOn()
		@@tmpstderr.close()
		STDERR.reopen( @@stderr  )
	end
	# Add the result of stderr to the log-File
	def add_stderr( tool )
		Logger.add(9, tool, ">>> Start messages (stderr) #{tool}>>>")
		Logger.add_list( get_stderr(), tool)
		Logger.add(9, tool, "<<< End messages (stderr) #{tool}<<<")
	end
	def get_stderr( )
		@@tmpstderr.open()
		stderr = @@tmpstderr.readlines()
		@@tmpstderr.close()
		return stderr
	end
end
#
#This class allows som precompile on the given TeX-File(s)
#Actual a method for listings.sty is implemented.
class Precompile
	#This method overwrites the given file with the replacements for listings
	def Precompile.mix_references!( filename )
	
		Logger.add(2, '', "Precompile #{filename} for cross-references")
		pre = new( filename)
		pre.listings_mix_references()
		
		new = File.new(filename, 'w')
		new.puts pre.text()	#write the converted text
		new.close	
	end	#mix_references( template )
	#Read the contents of the file
	def initialize( filename)
		@text = ''
		File.open( filename ){ |f|
			@text = f.readlines.to_s
		}
	end
	attr_reader :text
	#This function supports the handling of the package listings.
	#Listings.sty allows you the insertion of external lsitings with the command
	#	\lstinputlisting[firstline=<first>,lastline=<last>]{Programm}
	#There is one Problem: Whats the value of <first> and <last>?
	#Once a programm is finished, it is no problem. But when the program is still in develpment,
	#you have to adapt the line numbering.
	#It would be much nicer, if you could use something like 
	#	\lstinputlisting[fromlabel=<first>,tolabel=<last>]{Programm}
	#Well, I understand why it is dificult to make it in listings.sty.
	#This method makes this work. It scans the listing file for the label and replaces it in the source.
	def listings_mix_references( template = @text )
		#This method scan a program file for the string TeXlabel{<label>} or LABEL{<label>} and stores the linenumber.
		#The string can be redefined by a regular expression, $1 must be the label.
		def scan_source( source, r = /[(TeXlabel)|(LABEL)]\{(.*?)?\}/ )
			Logger.add(4, self.class, "Read File #{source} for cross-references")
			labels	= Hash.new( 0 )
			if !File.exist?( source )
				Logger.add(3, self.class,  "Sourcecode #{source} nicht gefunden" )
				return labels
			end
			#extend this Hash with a method get_label
			File.open(source){ |f|
				linenumber = 0
				f.each{|l|
					linenumber += 1
					while l =~ r	#Perhaps there are mor then one label on a line
						labels[$1] = linenumber
						l.sub!( r ,'')	#delete label for next Regex-check
					end
			#Idee fr ABAP: FORM/Endform automatisch mit Labels versehen?
				}
			}
			Logger.add(5, self.class,  "Found labels in #{source}: #{labels.inspect}")
			return labels
		end 
		def get_label( hash, key, source )
			pos = hash.fetch(key, 0)
			Logger.add(3, self.class,  "Label #{key} not found in #{source}") if pos == 0
			return pos
		end
		#If this script is running more the once, the inputlistings would increase.
		#So we clear it here to a "virgin" state.
		@text.gsub!( 
			/%(lstinputlisting\[.*?\]\{(.*?)\})\n.lstinputlisting\[.*?\]\{.*?\}/m,
			'\\\\\1'
			)
		#\ABAPinput is a macro used in a privat package of the author ;-)
		@text.gsub!( 
			/%(ABAPinput\[.*?\]\{(.*?)\})\n.ABAPinpu\[.*?\]\{.*?\}/m,
			'\\\\\1'
			)
		
		sources = Hash.new()	#all Sources
		#Replace all from/tolabe-listings with the new  first/lastline version.
		#Remember the original labels in a TeX-comment (needed next time, the programm could change.
		r = Regexp.new('\\\\(lstinputlisting|ABAPinput)\[.*(?:fromlabel=(.*?))(,.*)*(?:tolabel=(.*?))(,.*)*\]{(.*)}')
		while m = r.match(@text)
			old	= m[0]
			source = m[6]
			Logger.add(5, self.class, "Found lstinputlisting #{source}")
			if ! sources.has_key?(source)
				sources[source] = scan_source( source )
			end
			pos1 = get_label(sources[source], m[2], source ) + 1
			pos2 = get_label(sources[source], m[4], source ) - 1
			new	= '\\' + m[1] + '[' + 'firstline=' + pos1.to_s + m[3] +
					'lastline=' + pos2.to_s + m[5].to_s + "]{" + source  + '}'
			Logger.add(5, self.class,  "Command old:\t#{old}\nCommand new\t#{new}" )
			@text = m.pre_match + "%#{old[1..-1]}\n" +	new + m.post_match
		end
		return @text
	end	#mix_references
end	#Precompile

# This class defines a process chain to translate a tex-File.
# A process chain must contain all Tools from TeX to the target.
# The default process is pdflatex with assigned tools.
# Postprocessors (dvi2ps, ps2pdf...) can be integrated here
class Chain < Tool
	def initialize( filename=nil )
		if filename != nil
			self.file= filename
		end
		@tools = [ PDFLaTeX ]
	end
	attr_reader :filename
	#Define the TeX-file 
	def file=( filename )
		Dir.chdir(File.dirname(filename))
		@filename = File.basename( filename, ".tex" )
		if ! File.exist?( @filename + '.tex' )
			Logger.error "File #{@filename + '.tex'} not found (looking in #{Dir.pwd()})"
		end
	end
	# Create a DVI-File from LaTeX
	def latex()
		@tools = [ LaTeX ]
	end
	# Create a PDF-File with pdfLaTeX
	def pdflatex()
		@tools = [ PDFLaTeX ]
	end
	# Create a Postscript-File from LaTeX
	def latex_ps()
		@tools = [ LaTeX, DVIPS ]	
	end
	# Create a PDF from LaTeX via Postscript-File 
	def latex_ps_pdf()
		latex_ps()
		@tools << PS2PDF
	end
	#Run htlatex to create HTML
	def htlatex()
		@tools = [ HTLaTeX ]
	end
	def execute()
#		Logger.add( 0, File.basename($0), "Executing in #{Dir.pwd()}")
		if !@filename 
			Logger.error "rtex.rb: No File available"			
		elsif !File.file?(@filename  + '.tex')
			Logger.error "File #{@filename + '.tex'} not found"			
		else
			puts "Process #{filename} (Error-Level #{Logger.level()})"
			executeTools()
		end
	end
	# Create a PDF-File from LaTeX
	def latex_pdf()
		@tools = [ LaTeX, DVIPDF ]	
	end
end	#Chain

# The package rail.sty requires a run of rail.exe.
# If a rail-diagramms changed (or the sequence of different diagrams) the program must run again.
class Rail < Tool
	def initialize( filename )
		super()	#Set flags and tools
		@filename = filename
		#~ @logInclude	= false	#Add log-Files
		#~ @stdoutInclude	= true	#Add the stdout-Files
	end
	def rerun?( log = nil )
		result = false
		if log == nil; log = readLog() end
		if /Package: rail / =~ log	#Package: rail 1998/05/19
			Logger.add( 4, "Rail", "Package rail detected" )
		else
			Logger.add( 5, "Rail", "Package rail not used" )
			return false
		end
		log.each{ |l|
		if /Package rail Warning: Railroad diagram \{(\d*)\} doesn't match on input line (\d*)./ =~ l
			result = true
			Logger.add( 5, "Rail", "Rerun necessary (Rail diagram #{$1} on line #{$2})")
		elsif /Package rail Warning: Railroad diagram\(s\) may have changed./ =~ l
			result = true
			Logger.add( 5, "Rail", "Rerun necessary (Railroad diagram changed)")
		end
		}
		return result
	end
	def execute()
		if ! File.exist?( @filename + '.rai')
			Logger.error( "Rail: .rai-File is missing" )
		end
		if File.exist?( @filename + '.rao')
			raoold = File.new(@filename + '.rao').readlines()
		end
		@dataChanged = false	#clear flag
		#Rail.exe don't like long filenames, but via pipe it works
		Logger.add( 5, "Rail", "Execute #{Config.rail()}")
		stdoutOff( ) 
		stderrOff( ) 
		result = system( Config.rail("-t < #{@filename}.rai > #{@filename}.rao") )
		stderrOn( ) 
		stdoutOn( ) 
		# Delete first two lines from rao-File (contains Version-Message)
		rao = File.new( @filename + ".rao" ).readlines()
		rao.each{ |l|
			l.sub!(/^(This is Rail.*)/, '%\1')
			l.sub!(/^(\(stdin.*)/, '%\1')
			l.sub!(/^\)/, '%\1)')		#last line
			}
		rao2 = File.new( @filename + ".rao", 'w' )
		rao2.puts(rao)
		rao2.close()
		put_analyse( analyse( rao ) )
#		add_stdout( self.class ) if @stdoutInclude	#was redirected to @filename+'.rao'
		add_stderr( self.class ) if @stderrInclude
		Logger.add_file( "filename" ) if @logInclude
		if File.exist?( @filename + '.rao')
			raonew = File.new(@filename + '.rao').readlines()
			if raoold != raonew
				@dataChanged = true
			end
		else
			Logger.error( "Rail: .rao-File is missing" )
		end
		return result
	end
	def analyse( rao = [] )
		result = {	'Error'		=> 0,
				'Warning'		=> 0,
				'Entries'		=> 0,
				'Informationlist'	=> [],
				'Errorlist'	=> [],
				'Warninglist'	=> [],
				}
		get_stderr().each{ |l|
			if /stdin, line (.*): (.*)/ =~ l
				result['Error'] += 1
				result['Errorlist'] << "Line #{$1}: #{$2}"
			end
		}
		readLog( @filename + '.rao').each{|l|	#stdout was redirected
		#get_stdout().each{ |l|
			if /This is Rail, Version (.*)\s/ =~ l
				result['Informationlist'] << "Rail version #{$1} was called"
			elsif /(Der Prozess kann nicht auf die Datei zugreifen, .*)/ =~ l
				result['Error'] += 1
				result['Errorlist'] << "#{$1}"
			end
		}
		return result
	end
end	#Rail

#Actions to handle bibliographies with bibTeX.
#Glossaries with gloss.sty ...Bibliographies 
class BibTeX < Tool
	def initialize( filename, id=nil )
		super()
		@filename	= filename
		@id = self.class.to_s
		@id += "/" + id if id
	end
	def BibTeX.create( filename, myclass=BibTeX )
		result = []
		# Check Standard log:
		log = Tool.readLog( filename + '.aux')
		if /bibdata/ =~ log
			Logger.add( 5, 'BibTeX', "Bibliography detected")
			result << BibTeX.new( filename )
		end
		# Check gloss.sty
		log = readLog( filename + '.log')
		log.each{ |l|
#			if /Package gloss Info: Writing gloss file (.*).aux on input line .*/ =~ l
			if /Package gloss Info: Writing gloss file (.*).aux/ =~ l
				Logger.add( 5, 'BibTeX', "Bibliography #{$1} detected")
				result << BibTeX.new( $1, $1 )
			end
		}
		if result == []
			Logger.add( 5, 'BibTeX', "No bibliography detected" )
		end
		return result
	end
	def rerun?( log = nil )
		return true	#rerun every times
	end
	def execute()
		result = true
		@log = ''
		@dataChanged = false	#clear flag
		Logger.add( 5, self.class, "Execute #{Config.bibtex}for \"#{@filename}\"" )
		if File.exist?( @filename + '.bbl')
			bblold = File.new(@filename + '.bbl').readlines()
		end
		cmd = Config.bibtex(@filename)
		Logger.add( 5, self.class, "Cmd: \"#{cmd}\"" )
		stdoutOff()
		result = system( cmd )
		stdoutOn()
		add_stdout( self.class ) if @stdoutInclude
		add_stderr( self.class ) if @stderrInclude
		Logger.add_file( @filename + '.blg' ) if @logInclude
		if !result
			Logger.add( 1, self.class, "Error executing #{Config.bibtex} #{@filename}")
			#Logger.error( "BibTeX: Error during execution" )
			#@log = get_stdout().to_s
		end
		@log += readLog( @filename + '.blg' )
		put_analyse( analyse( ) )
		if File.exist?( @filename + '.bbl')
			bblnew = File.new(@filename + '.bbl').readlines()
			if bblold != bblnew
				@dataChanged = @dataChanged and true
			end
		else
			Logger.error( "BibTeX: #{@filename}.bbl-File is missing" )
		end
		return result
	end
	def analyse( )
		result = {	'Error'		=> 0,
				'Warning'		=> 0,
				'Informationlist'	=> [],
				'Errorlist'	=> [],
				'Warninglist'	=> [],
				}
		@log.each{ |l|
			if /Database file #(.*): (.*)/ =~ l
				result['Informationlist'] << "Database file #{$2} used"
			elsif /Warning--I didn't find a database entry for "(.*)"/ =~ l
				result['Warning'] += 1
				result['Warninglist'] << "Databaseentry #{$1} missing"
			#~ elsif /Warning--Empty definition in (.*)/ =~ l
				#~ result['Warning'] += 1
				#~ result['Warninglist'] << "Empty definition #{$1}"
			elsif /Warning--(.*)/ =~ l
				result['Warning'] += 1
				result['Warninglist'] << "#{$1}"
			elsif /I couldn't open (.*) file (.*)/ =~ l
				result['Error'] += 1
				result['Errorlist'] << "#{$1} #{$2} not found"
			elsif /I found no (.*)---while reading file(.*)/ =~ l
				result['Error'] += 1
				result['Errorlist'] << "found no #{$1} in #{$2}"			
			elsif /(.*)---line (.*) of file(.*)/ =~ l
				result['Error'] += 1
				#line-number ist unsinnig
				result['Errorlist'] << "#{$1} in #{$3}"
			end
		}
		return result
	end
end	#BibTeX

#
# Makeindex is used by the standard LaTeX and by index.sty (part of camel)
#
class Makeindex< Tool
	def initialize( filename, source=filename+'.idx', target=filename+'.ind' )
		super()
		@filename	= filename
		@source	= source
		@target	= target
		@id		= self.class.to_s
		@id		+= '/' + @source if @source !=filename+'.idx'
		@logInclude	= true	#Add log-Files
		@stdoutInclude	= false	#Add the stdout-Files
		@stderrInclude	= false	#Add the stderr-Files same as ilg-file
	end
	def Makeindex.create( filename, myclass=Makeindex )
		result  = []	
		indexstymessage = true	#Flag if the message was given
		log = Tool.readLog( filename + '.log')	#TeX-Log
		log.each{ |l|
			#This message is posted, if index2 is used.
			#I hope the responsible person for index.sty will take my changes.
			#If the name is very long, there is a break in the Filename.
#			if /Please call "makeindex -o (.*) (.*)"/ =~ l
			if /Package index2 Info: (.*) (.*)/ =~ l
				Logger.add( 5, "Makeindex", "Index #{$1} detected (->#{$2})")
				Logger.add( 2, "Makeindex", "Warning: Name #{$2} perhaps cutted") 	if $1.size != $2.size
				result << Makeindex.new(filename, $1, $2) # if $2 != filename + '.ind'
				indexstymessage = false
			#elsif /Writing index file (.*) on.*/ =~ l
			elsif /Writing index file (.*)/ =~ l
				Logger.add( 5, "Makeindex", "Index #{$1} detected")
				#~ puts filename + '.idx', $1
				if $1 == filename + '.idx'
					result << Makeindex.new(filename, $1, filename + '.ind')
				else
					Logger.add( 0, "Makeindex",  "index.sty is not supported, please use index2 (#{$1})") if indexstymessage 
					indexstymessage = false
				end
			end
		}
		if result == []
			Logger.add( 5, "Makeindex", "No index detected" )
		end
		return result
	end
	def rerun?( log = nil )
		result = true
	end
	def execute()
		@dataChanged = false	#clear flag
		Logger.add( 5, self.class, "Execute #{Config.makeindex()} for #{@source}")
		if File.exist?( @target )
			indold = File.new( @target ).readlines()
		end
		cmd = Config.makeindex( "-o #{@target} #{@source}" )
		stderrOff()
		result = system(cmd)
		stderrOn()
		@log = Tool.readLog( @filename + '.ilg' )
		put_analyse( analyse( ) )
		add_stdout( self.class ) if @stdoutInclude
		add_stderr( self.class ) if @stderrInclude
		Logger.add_file( @filename + '.ilg' ) if @logInclude
		if !result
			Logger.add( 1, self.class, "Error executing #{Config.makeindex}")
		end
		if File.exist?( @target )
			indnew = File.new( @target ).readlines()
			if indold != indnew
				@dataChanged = true
			end
		else
			Logger.error( "makeindex: #{@target}-File is missing" )
		end
		return result
	end
	def analyse( )
		result = {	'Error'		=> 0,
				'Rejected'	=> 0,
				'Warning'		=> 0,
				'Informationlist' => [],
				'Errorlist'	=> [],
				'Warninglist'	=> [],
				}
		flag		= 0; 	errorpos	= ""
		input = [];	inputfile	= ""
		@log.each{ |l|
			if flag == 1	#Index error
				result['Errorlist'] << "#{l.chomp.sub(/.*--/, "")} #{errorpos}"
				flag		= 0; 	errorpos	= ""
				next
			elsif /Scanning input file (.*)\...done \((.*) entries accepted, (.*) rejected\)./ =~ l
				inputfile = $1
				input = File.new(inputfile).readlines() if File.exist?(inputfile)
				result['Informationlist'] << "input file #{$1} (#{$2} entries accepted, #{$3} rejected)"
			elsif /done \((.*) entries accepted, (.*) rejected\)./ =~ l
				result['Rejected'] += $2.to_i() if $2.to_i() > 0
			elsif /!! Input index error \(file = (.*), line = (.*)\):/ =~ l
				result['Error'] += 1
				flag = 1	#next line contains error
				entry = input[$2.to_i - 1] if $1 == inputfile
				entry.chomp!() if entry
				errorpos = "(file #{$1} line #{$2}: #{entry})"
			end
		}
		return result
	end
end	#Makeindex

# This class provides (pdf)LaTeX.
# Tools which support LaTeX are in the array @tools and are called automatic
class LaTeX < Tool
	def initialize( filename )
		super()
		@filename = File.basename( filename, ".tex" )
		if ! File.exist?( @filename + ".tex" )
			Logger.error "File #{@filename + '.tex' } not found"
		end
		@cmd = Config.latex	#may be overloaded
		@logInclude	= false	#Add log-Files
		@stdoutInclude	= true	#Add the stdout-Files
		@tools		= [ Rail, BibTeX, Makeindex ]
		@first		= true	#needed for rerun?
	end
	
	#Easy check: If the data changed (aux-File) a rerun is necesary.
	#With this method, one run too much is done.
	#A bit more complex:
	#1. Check the log-File if an additional run is necessary.
	#	- references
	#	- longtable.sty: balancing not finished
	#2. Check different Tools if they must be active and if they force an additional run
	#	- BibTeX (bibliography)
	#	- makeindex (including multiind)
	#	- rail-Diagramms
	def rerun?( log = nil )
		if @first
			@first = false
			return true
		#~ else 
			#~ return @dataChanged	#results in a additional run
		end
		result = false	#no automatic rerun
		error	= false
		if log == nil;	log = readLog()	end
		# Check reasons for another run
		if /Package hyperref Warning: old toc file detected, not used; run LaTeX again/ =~ log
			result = true
			Logger.add( 6, self.class, "Rerun necessary (old toc)")
		end
		if /LaTeX Warning: Label(s) may have changed. Rerun to get cross-references right./ =~ log
			result = true
			Logger.add( 6, self.class, "Rerun necessary (Labels/Toc changed)")
		end
		if /Package longtable Warning: Table widths have changed. Rerun LaTeX./ =~ log
			result = true
			Logger.add( 6, self.class, "Rerun necessary (Longtable requires additional calculation)")
		end

		if executeTools()
			result = true
		end
		
		if result == true and Logger.texrun() >= Config.maxrun
			Logger.add( 0, self.class, "Abort after #{Logger.texrun()} runs" )		
			result = false
		elsif result == false
			Logger.add( 6, self.class, "No reason for additional run detected" )
		end
		if @dataChanged and ! result
			#@dataChanged = false	#if not changed, execute_tools reports the need of an additional run
			result = true
			Logger.add(5, self.class, "The data (aux-File) changed, but no reason for an additional run found")
		end
		if result
			Logger.add( 5, self.class, "Rerun necessary" )		
		end
		return result
	end
	
	# Execute TeX/LaTeX/PDFLaTeX... as often as necessary.
	def execute()
		@logInclude	= true	#Add log-Files
		@stdoutInclude	= false	#nearly empty in batchmode
		cmd = "#{@cmd} -interaction=batchmode " + @filename
		first = true
		#~ @logInclude	= false	#Add log-Files
		#~ @stdoutInclude	= true	#Add the stdout-Files
		#~ cmd = "#{@cmd} " + @filename
		while rerun? or first
			if first
				first = false
			else
				Logger.nextrun()
			end
			if File.exist?( @filename + '.aux')
				auxold = File.new(@filename + '.aux').readlines()
			end
			@dataChanged = false	#clear flag
			Logger.add( 5, self.class, "Execute #{@cmd}")
			stdoutOff()	#Messages
			stderrOff()	#included pictures 
			system cmd
			stdoutOn()
			stderrOn()
			if File.exist?( @filename + '.aux')
				auxnew = File.new(@filename + '.aux').readlines()
				if auxold != auxnew
					@dataChanged = true
				end
			else
				Logger.error( "#{self.class}: .aux-File is missing" )
			end
			result = put_analyse( analyse() )
			Logger.add(1, self.class, result['Summary'])
			add_stdout( self.class )	if @stdoutInclude
			add_stderr( self.class )	if @stderrInclude
			Logger.add_file( @filename + '.log' ) if @logInclude
		end
		if result['Error'] > 0 
			return false
		else
			return true
		end
	end
	def	convertLog( log )
		# determine actual output page
		return log
		puts "log-analyse Seite File erkennen offen"
		#fixme: [x.x] 
		#log2 = log.gsub(/(\[\d+)/m, "\n" + '\1' + "<<<page\n" )
		# determine actual source file 
		log2 = log.gsub(/\((\..*?)\s/m, "(\n>>>>" + '\1' + ">>>>\n" )	#recognize start
		log2 = log.gsub(/\(((?:(?:\.)|(?:c:\/)|(?:\/)).+?)([\s\{])/m, 
				"(\n>>>>" + '\1' + ">>>>\n"
				)
		#~ log2 = log.gsub(/\(((?:(?:\.)|(?:c:\/)|(?:\/)).+?)([\s\{])(.*?)\)\s/m, 
				#~ "(\n>>>>" + '\1' + ">>>>\n" + '\2\3' + "\n<<<<" + '\1' + "<<<<\n)"
				#~ )
		#Idee: jede neue Seite / File in eigene log-Zeile. Dort fortschreiben
		f = File.new( 'c:\temp\log.log', 'w')
		f.puts(log2)
		f.close
	end
	def analyse( log = readLog() )
		result = {	'Error'		=> 0,
				'Warning'		=> 0,
				'Overfull'	=> 0,
				'Underfull'	=> 0,
				'Errorlist'	=> [],
				'Warninglist'	=> [],
				'WarninglistHyper'	=> [],
				'Labellist'	=> [],
				'Citationlist'	=> [],
				'Overfulllist'	=> [],
				'Underfulllist'	=> [],
				}
		flag	= nil
		file	= ""	#Container for actual file
		listtext	= ""	#Container for error-text
		list	= "" #Container for list
		convertLog( log )
		log.each_line{ |logline|
			#~ if /\((\..*?)\s/m =~ logline
				#~ file = $1
				#~ puts file
			#~ end
			if flag == 'n'	#add next line to list
				# jump over some Informations
				if	logline == "See the LaTeX manual or LaTeX Companion for explanation.\n" or
					logline == "Type  H <return>  for immediate help.\n" or
					/^\s*\n/	=~ logline  or	#empty line
					/\s*\.\.\.\s*\n/ =~ logline	#...
					next
				end
				# Add line
				flag = nil
				if /l.([0-9]*) (.*)/ =~ logline
					result[list] << "#{listtext} (#{$2} on line #{$1})"
				else
					result[list] << "#{listtext} (#{logline.chomp()})"
				end
				listtext = list = ""
			#	Errors where the message continues on next line
			elsif	/^!\s*(LaTeX Error:)\s*(.*)/ =~ logline
				result['Error'] += 1
				list = 'Errorlist'
				listtext = $2
				flag = 'n'
				#~ result['Errorlist'] << "? #{$2}"
				#~ result['Error'] += 1				 
			elsif	/^!\s*(Undefined control sequence.)/ =~ logline or 
				/^!\s*(Dimension too large.)/ =~ logline
				# report error with next line
				result['Error'] += 1
				list = 'Errorlist'
				listtext = $1	#logline.chomp
				flag = 'n'
			elsif	/(job aborted, no legal \\end found)/ =~ logline
				result['Errorlist'] << $1
				result['Error'] += 1
			elsif	/Package (.*) Error: (.*)/ =~ logline
				result['Errorlist'] << "#{$1}.sty:\t#{$2}"
				result['Error'] += 1
			elsif	/^! pdfTeX warning .*?: (.*)/ =~ logline
				result['Warninglist'] << $1
				result['Warning'] += 1
			elsif	/^!\s*(.*)/ =~ logline	#really everytimes?
				#~ /(! Emergency stop.)/ =~ logline or
				#~ /(! Huge page cannot be shipped out.)/ =~ logline 
				result['Errorlist'] << $1
				result['Error'] += 1
			elsif	/LaTeX Warning: Label (.*)/ =~ logline
				result['Labellist'] << $1
				result['Warning'] += 1
			elsif	/LaTeX Warning: (Reference .*)/ =~ logline
				result['Labellist'] << $1
				result['Warning'] += 1
			elsif	/LaTeX Warning: Citation (.*)/ =~ logline
				result['Citationlist'] << $1
				result['Warning'] += 1
			elsif	/LaTeX Warning: (.*)/ =~ logline
				result['Warninglist'] << $1
				result['Warning'] += 1
				#fixme: add next line, if indent is title?
				#~ LaTeX Warning: You have requested package `Blindtext',
				#~ but the package provides `blindtext'.
			#~ elsif	/Package hyperref Warning: (.*)/ =~  logline
			elsif	/Package hyperref Warning: (.*)/ =~ logline
				result['WarninglistHyper'] << "#{$1}"
				result['Warning'] += 1
			elsif	/Package (.*) Warning: (.*)/ =~ logline
				result['Warninglist'] << "#{$1}:\t#{$2}"
				result['Warning'] += 1
			elsif	/Overfull \\hbox \((.*) too wide\) in paragraph at lines (.*)/ =~ logline
				next if $1.to_i() < Config.overfull
				flag = 'n'	#add next line in logfile
				list = 'Overfulllist'
				listtext = "#{$1} at lines #{$2}"
				result['Overfull'] += 1
			elsif	/Underfull (.*box) \(badness (.*)\) detected at line (.*)/ =~ logline
				next if $1.to_i() < Config.underfull
				flag = 'n'	#add next line in logfile
				list = 'Underfulllist'
				listtext = "badness #{$2} at lines #{$3}"
				result['Underfull'] += 1
			#~ elsif	// =~ logline
				#~ result['Errorlist'] << $1
				#~ result['Error'] += 1
			elsif /Output written on (.*) \((.*) pages?, (.*) bytes\)/ =~ logline
				result['File']	= $1
				result['Pages']	= $2
				result['Size']	= "#{$3.to_i / 1024}KB"
			#~ elsif /Output written on (.*) \((.*) page, (.*) bytes\)/ =~ logline
				#~ result['File']	= $1
				#~ result['Pages']	= $2
				#~ result['Size']	= "#{$3.to_i / 1024}KB"
			end
		}
		return result
	end
end	#LaTeX

class PDFLaTeX < LaTeX
	def initialize( filename )
		super( filename )
		@cmd = Config.pdflatex
	end
end	#PDFLaTeX

class DVIPS< Tool
	def initialize( filename )
		super()
		@filename = filename
		@logInclude	= false	#Add log-Files
		@stdoutInclude	= false	#Add the stdout-Files
		@stderrInclude	= true	#Add the stderr-Files same as ilg-file
		@first		= true
	end
	def rerun?( log = nil )
		return @first	#Postprocessor runs only once
	end
	def execute()
		@first		= false
		Logger.add( 5, self.class, "Execute #{Config.dvips}")
		@dataChanged = false	#clear flag
		stderrOff()
		result = system( Config.dvips(@filename) )
		stderrOn()
		@log = get_stderr()
		put_analyse( analyse( @log ) )
		if !result
			Logger.add(1, self.class, "Error executing #{self.class}")
		end
		add_stdout( self.class ) if @stdoutInclude
		add_stderr( self.class ) if @stderrInclude
		return result
	end
	def analyse( log=@log )
		result = {	'Error'		=> 0,
				'Warning'		=> 0,
				'Informationlist' => [],
				'Errorlist'	=> [],
				'Warninglist'	=> [],
			}
		log.each{ |l|
			if /This is (.*) Copyright/ =~ l
				result['Informationlist'] << "Called #{$1}"
			# check analyses
			elsif /[Ee]rror/ =~ l
				result['Error'] += 1
				result['Errorlist'] << "#{l}"
			elsif /[Ww]arning/ =~ l
				result['Warning'] += 1
				result['Warninglist'] << "#{l}"
			end
		}
		return result
	end
end	#DVIPS

class PS2PDF< Tool
	def initialize( filename )
		super()
		@filename = filename
		@logInclude	= false	#Add log-Files
		@stdoutInclude	= true	#Add the stdout-Files
		@stderrInclude	= false	#Add the stderr-Files 
		@first		= true
	end
	def rerun?( log = nil )
		return @first	#Postprocessor runs only once
	end
	def execute()
		@first		= false
		Logger.add( 5, self.class, "Execute #{self.class} (#{Config.ps2pdf})")
		@dataChanged = false	#clear flag
		stdoutOff()
		result = system( Config.ps2pdf( @filename ) )
		stdoutOn()
		@log = get_stdout()
		put_analyse( analyse( @log ) )
		if !result
			Logger.add(1, self.class, "Error executing #{self.class}")
		end
		add_stdout( self.class ) if @stdoutInclude
		add_stderr( self.class ) if @stderrInclude
		return result
	end
	def analyse( log=@log )
		result = {	'Error'		=> 0,
				'Warning'		=> 0,
				'Informationlist' => [],
				'Errorlist'	=> [],
				'Warninglist'	=> [],
				}
		log.each{ |l|
			if /Unable to open the initial device, quitting./ =~ l
				result['Error'] += 1
				result['Errorlist'] << "Unable to open the initial device (PDF opened in Acrobat?)"
			elsif /[Ee]rror/ =~ l
				result['Error'] += 1
				result['Errorlist'] << "#{l}"
			elsif /[Ww]arning/ =~ l
				result['Warning'] += 1
				result['Warninglist'] << "#{l}"
			end
		}
		return result
	end
end	#PS2PDF

class DVIPDFM< Tool
	def initialize( filename )
		super()
		@filename = filename
		@logInclude	= false	#Add log-Files
		@stdoutInclude	= true	#Add the stdout-Files
		@stderrInclude	= false	#Add the stderr-Files 
		@first		= true
	end
	def rerun?( log = nil )
		return @first	#Postprocessor runs only once
	end
	def execute()
		@first		= false
		Logger.add( 5, self.class, "Execute #{self.class} (#{Config.ps2pdf})")
		@dataChanged = false	#clear flag
		stdoutOff()
		result = system( Config.dvipdfm( @filename ) )
		stdoutOn()
		@log = get_stdout()
		put_analyse( analyse() )
		if !result
			Logger.add(1, self.class, "Error executing #{self.class}")
		end
		add_stdout( self.class ) if @stdoutInclude
		add_stderr( self.class ) if @stderrInclude
		return result
	end
	def analyse( log=@log )
		result = {	'Error'		=> 0,
				'Errorlist'	=> [],
				'Warninglist'	=> [],
				}
		log.each{ |l|
			if /Unable to open the initial device, quitting./ =~ l
				result['Error'] += 1
				result['Errorlist'] << "Unable to open the initial device (PDF opened in Acrobat?)"
			elsif /[Ee]rror/ =~ l
				result['Error'] += 1
				result['Errorlist'] << "#{l}"
			elsif /[Ww]arning/ =~ l
				result['Warning'] += 1
				result['Warninglist'] << "#{l}"
			end
		}
		return result
	end
end	#DVIPDFM

#HTLaTeX to create HTML-Files
class HTLaTeX < Tool
	def initialize( filename )
		super()
		@filename = File.basename( filename, ".tex" )
		if ! File.exist?( @filename + ".tex" )
			Logger.error "File #{@filename + '.tex' } not found"
		end
		@logInclude	= false	#Add log-Files
		@stdoutInclude	= true	#Add the stdout-Files
		@tools		= [ ]
		@first		= true	#needed for rerun?
	end
	def rerun?( log = nil )
		if @first
			@first = false
			return true
		end
		result = false	#no automatic rerun
		error	= false
		return result
	end
	
	def execute()
		@logInclude	= true	#Add log-Files
		@stdoutInclude	= false	#nearly empty in batchmode
		cmd = Config.htlatex( @filename )
		@dataChanged = false	#clear flag
		Logger.add( 2, self.class, "Execute #{cmd}")
		stdoutOff()
		system cmd
		stdoutOn()
		Logger.add( 2, self.class, "Executed #{cmd}")
		result = put_analyse( analyse( get_stdout ) )
#		Logger.add(1, self.class, result['Summary'])
		add_stdout( self.class )	if @stdoutInclude
		add_stderr( self.class )	if @stderrInclude
#		Logger.add_file( @filename + '.log' ) if @logInclude
		if result['Error'] > 0 
			return false
		else
			return true
		end
	end
	# -> not implemented
	def analyse( log = get_stdout() )
		result = {	'Error'		=> 0,
				'Warning'		=> 0,
				'Overfull'	=> 0,
				'Underfull'	=> 0,
				'Errorlist'	=> [],
				'Warninglist'	=> [],
				'Labellist'	=> [],
				'Citationlist'	=> [],
				'Overfulllist'	=> [],
				'Underfulllist'	=> [],
				}
		Logger.add( 3, self.class, "Analyse HTLaTeX not made")
		#htlatex calls LaTeX -> same analyse
		tex	= LaTeX.new( @filename )
		result = tex.analyse( readLog() ) 
		
		log.each{ |l|
			puts l	#fixme
		}
		result['Warning'] += 1
		result['Warninglist'] << "rtex.rb: no analyses for htlatex available"
		return result
	end
end	#HtLaTeX

#
# Set parameter in a gui and start then.
# - make a screen for each tool?
def gui( tex )
	if !tex.is_a?(Chain)
		raise 'Internal error: Gui get no Chain'
	end
	ph = {'padx' => 10, 'pady' => 10 }
	#p = proc{ puts @text.value }
	
	root	= TkRoot.new{ title "rtex: TeX-Cockpit" }
	top	= TkFrame.new(root)
	@text = TkVariable.new
	@text.value=tex.filename + '.tex'
	@text.pack( top, ph )
	#tex.filename is the file without extension
	TkLabel.new(top) {text	tex.filename + '.tex'; pack(ph) }
	#~ @entry	= TkEntry.new(top, 'textvariable' => @text )
	#~ @entry.pack(ph)
	#~ TkButton.new(top) { text 'Start';	command proc{ tex.execute() }; pack ph}
	TkButton.new(top) { text 'Exit';	command proc{exit}; pack ph } #+ { 'side' => 'left' } }
	top.pack('fill'=>'both', 'side'=>'top')
	
	Tk.mainloop
end #gui

#Flag, if the program should ask for a confirmation before ending
@@wait = false
Config.logfile	= 'runtex.log'

#Analyse from ARGV when called from Desktop
def start_from_desktop( )
	#Create the logger and initialize the program
	log	= Logger.create()
	run	= Chain.new()
	run.pdflatex()
	gui	= false
	
	opt	= GetoptLongWithHelp.new(
			[ "--level",	'-l', GetoptLong::REQUIRED_ARGUMENT ],
			[ "--action",	'--chain', 
						'-a', GetoptLong::REQUIRED_ARGUMENT ],
			[ "--overfull", '-o',GetoptLong::REQUIRED_ARGUMENT ],
			[ "--underfull", '-u',GetoptLong::REQUIRED_ARGUMENT ],
			[ "--precompile", '-p',GetoptLong::REQUIRED_ARGUMENT ],
			[ "--nohyperrefwarnings", GetoptLong::NO_ARGUMENT ],
			[ "--gui",		'-g',GetoptLong::NO_ARGUMENT ],
			[ "--wait",	'-w', GetoptLong::NO_ARGUMENT ]
		)
	opt.addHelp2('--level', %Q|set the level of informations you get.
		0 No information at all
		1 Tool-Error + Summary information from TeX
		2 Level 1 + Summary information from tools
		3 Level 2 + Executed steps + Statistics + Errors
		4 Level 3 + Warnings
		5 Level 4 + hints from process
		6 Level 5 + detailed hints from process
		9 Everything
		All levels are reported in the Log-File|.gsub(/^\t/, ''))
	opt.addHelp2( '-a', %Q|Define the action chain
		A chain defines the steps from source to the target.
		- pdfLaTeX	[default]
		- LaTeX
		- LaTeXPS	Call LaTeX and then create a 
				Postscript file (dvips)
		- LaTeXPSPDF	Call LaTeX and then create a PDF 
				via dvips and ghostscript
		-LaTeXPDF	Call LaTeX and then create a PDF with dvipdfm
		remark:	Each tool may define a "subchain"
			(pdf)LaTeX calls bibtex, makeindex, rail if necessary|.gsub(/^\t/, ''))
	opt.addHelp2( '-o',	"Set the limit, from which overfull boxes are reported")
	opt.addHelp2( '-u',	"Set the limit, from which underfull boxes are reported")
	opt.addHelp2( '-w',	"Wait after finishing the script\n\t(helpfull, when started by another programms)")
	opt.addHelp2( '-g',	"Call GUI" )
	opt.addHelp2( '-p',	%q|The given file(s) are precompiled.
		Conversions of the precompiler:
		- listings.sty:
			\lstinputlisting[fromlabel=<label>,tolabel=<label>]{file} to 
			\lstinputlisting[firstline=<num>,lastline=<num>] {file}
			file must contain tags like "TeXlabel{label}"
			
	|.gsub(/^\t/, ''))
	opt.mkHelp( %q|runtex.rb: Translate a TeX-File.
		This tool checks, how often the file must be translated to 
		create a document with correct references.
		Usage:
			runtex.rb [switches] source[.tex]
		Switches:|.gsub(/^\t\t/, ''),
	%q|-h <switch> for details of option <switch>
		Known Problems:
		- Not tested on different systems (PS2PDF will work only on windows)
		- The errors statistic count some errors twice (or do you want to loose errors)
		- index.sty is not supported. Use index2.sty instead 
			(I hope it is obsolete for the future)
		Detailed informations: http://www.lickert.net/dev/ruby/rtex|.gsub(/^\t\t/, '')
	)

	opt.each{ |option, selection|
		selection = selection.downcase()
		case option
		when '--help'
			puts opt.help( selection )
			STDIN.getc if @@wait
			exit
		when '--HELP'
			puts opt.help( '--HELP' )
			STDIN.getc if @@wait
			exit
		when '--level'
			log.level=selection.to_i
		when '--action'
			case selection 
			when 'latex'
				run.latex()
			when 'pdflatex'
				run.pdflatex
			when 'latexps'
				run.latex_ps
			when 'latexpdf'
				run.latex_pdf
			when 'latexpspdf'
				run.latex_ps_pdf
			when 'htlatex'
				run.htlatex
			else
				puts "Action chain #{selection} not valid"
				STDIN.getc if @@wait
				exit
			end	#case selection
		when '--overfull'	#set tolerance for overfull boxes
			Config.overfull= selection.to_i
		when '--underfull'	#set tolerance for underfull boxes
			Config.underfull= selection.to_i
		when '--nohyperrefwarnings'	#Display no warnings of hyperref
			Config.showhyperrefwarning= false
		when '--wait'	#Wait after running the script
			@@wait = true
		when '--precompile'
			selection.each{|f|
				Precompile.mix_references!( f )
				}
		when '--gui'	#call GUI for settings
			gui = true
		else
			puts "Option #{option}#{selection} not known\nUse -h for help"
			STDIN.getc if @@wait
			exit
		end	#case otion
	}
	if ARGV.size == 0
		#Message if called by exe created with rubyscript2exe
		puts "Add file to translate or -h for help" if /app.rb/ =~ $0
	end 
	ARGV.each{|arg|
			run.file=( arg )
	}

	puts "No TeX-File given" if !run.filename
	if gui
		run.file = Dir.pwd() + '/test.tex' if !run.filename	#fixme
		gui( run )
	else
		run.execute()
		log.save(Config.logfile)
	
		puts "Thanks for using #{File.basename($0)}."
		STDIN.getc if @@wait
	end

end	#start_from_desktop( )

start_from_desktop( ) if __FILE__ == $0



# >> Test! While testing the file c:\temp\test.tex is taken
#	Vorsicht. jedes run.file setzt den pwd um!
#	run.file = 'c:\temp\rtexTest\test.tex'	
#~ run.file = 'C:\usr\Vcd_Gem\Protokolle\VCDGem2003.tex'
#~ run.file=( 'C:\usr\Festo\Manualpages\PriceCatalog\Preiskatalog')
#~ run.latex 
#~ run.latex_ps_pdf
#log.level=5
# <<Test end
#~ run.execute()
#~ log.save(Config.logfile)
